      SUBROUTINE DCREC (X, Y, U, V)
C-----------------------------------------------------------------------
C             COMPLEX RECIPROCAL U + I*V = 1/(X + I*Y)
C-----------------------------------------------------------------------
      DOUBLE PRECISION X, Y, U, V
      DOUBLE PRECISION D, T
C
      IF (DABS(X) .GT. DABS(Y)) GO TO 10
         T = X/Y
         D = Y + T*X
         U = T/D
         V = -1.D0/D
         RETURN
   10 T = Y/X
      D = X + T*Y
      U = 1.D0/D
      V = -T/D
      RETURN
      END
      SUBROUTINE CDIVID (AR,AI,BR,BI,CR,CI)
C-----------------------------------------------------------------------
C     DOUBLE PRECISION COMPLEX DIVISION C = A/B AVOIDING OVERFLOW
C-----------------------------------------------------------------------
      DOUBLE PRECISION AR, AI, BR, BI, CR, CI
      DOUBLE PRECISION D, T, U, V
C
      IF (DABS(BR) .LE. DABS(BI)) GO TO 10
      T = BI/BR
      D = BR + T*BI
      U = (AR + AI*T)/D
      V = (AI - AR*T)/D
      CR = U
      CI = V
      RETURN
C
   10 IF (BI .EQ. 0.D0) GO TO 20
      T = BR/BI
      D = BI + T*BR
      U = (AR*T + AI)/D
      V = (AI*T - AR)/D
      CR = U
      CI = V
      RETURN
C
C     DIVISION BY ZERO. C = INFINITY
C
   20 CR = 1.797693134862316+308
      CI = CR
      RETURN
      END
      SUBROUTINE DCERFC (MO, Z, W)
C-----------------------------------------------------------------------
C
C              COMPUTATION OF THE COMPLEX COERROR FUNCTION
C
C                           ----------------
C
C           W = ERFC(Z)           IF MO = 0 OR REAL(Z) .LT. 0
C           W = DEXP(X*X)*ERFC(Z)  OTHERWISE 
c!!!!!      , 
c!!!!!  :       
C!!!!!       W = DEXP(Z*Z)*ERFC(Z)    MO = 1 
C
C-----------------------------------------------------------------------
      DOUBLE PRECISION Z(2), W(2)
      DOUBLE PRECISION M, N, N2, N4, NP1
      DOUBLE PRECISION C, C2, D, D2, E, EPS, R, SN, TOL, X, Y
      DOUBLE PRECISION A0(2), AN(2), B0(2), BN(2)
      DOUBLE PRECISION G0(2), GN(2), H0(2), HN(2)
      DOUBLE PRECISION QF(2), SM(2), SZ(2), TM(2), TS(2), W0(2), WN(2)
      DOUBLE PRECISION ANORM
      DOUBLE PRECISION DXPARG
      DATA DXPARG /-.70839641853155570D+03/
C------------------------
      ANORM(X,Y) = DMAX1(DABS(X),DABS(Y))
C------------------------
C     C = 1/SQRT(PI)
C------------------------
      DATA C /.56418958354775628694807945156077D0/
C------------------------
C
C     ****** EPS IS A MACHINE DEPENDENT CONSTANT. EPS IS THE
C            SMALLEST NUMBER SUCH THAT 1.D0 + EPS .GT. 1.D0.
C
      DATA EPS /2.220446049250313D-016/
C
C------------------------
      X = Z(1)
      Y = Z(2) 
      SN = 1.D0
      IF (X .GE. 0.D0) GO TO 10
         X = -X
         Y = -Y
         SN = -1.D0
C
   10 IF (MO .NE. 0  .AND.  SN .EQ. 1.D0  .AND.
     *   DMAX1(X, DABS(Y)) .GE. 144.D0) GO TO 100
      R = X*X + Y*Y
      SZ(1) = X*X - Y*Y
      SZ(2) = 2.D0*X*Y
C
      IF (R .LE. 1.D0) GO TO 20
      IF (R .GE. 144.D0) GO TO 100
      IF (DABS(Y) .GT. 31.8D0*X) GO TO 50
      IF (DABS(Y) .GT. 7.0D0*X .AND. R .LT. 64.D0) GO TO 50
      IF (DABS(Y) .GT. 3.2D0*X .AND. R .LT. 49.D0) GO TO 50
      IF (DABS(Y) .GT. 2.0D0*X .AND. R .LT. 36.D0) GO TO 50
      IF (DABS(Y) .GT. 1.2D0*X .AND. R .LT. 25.D0) GO TO 50
      IF (DABS(Y) .GT. 0.9D0*X .AND. R .LT. 16.D0) GO TO 50
      IF (R .GE. 6.25D0) GO TO 80
      IF (DABS(Y) .GT. 0.6D0*X) GO TO 50
      IF (R .GE. 4.0D0) GO TO 40
C
      D = X - 2.D0
      IF (D*D + Y*Y .LT. 1.D0) GO TO 40
      GO TO 50
C
C                          TAYLOR SERIES
C
   20 C2 = C + C
      TM(1) = C2*X
      TM(2) = C2*Y
      SM(1) = TM(1)
      SM(2) = TM(2)
      TOL = 2.D0*EPS
      M = 0.D0
   21    M = M + 1.D0
         D = M + M + 1.D0
         TS(1) = TM(1)*SZ(1) - TM(2)*SZ(2)
         TS(2) = TM(1)*SZ(2) + TM(2)*SZ(1)
         TM(1) = -TS(1)/M
         TM(2) = -TS(2)/M
         TS(1) = TM(1)/D
         TS(2) = TM(2)/D
         SM(1) = SM(1) + TS(1)
         SM(2) = SM(2) + TS(2)
         IF (ANORM(TS(1),TS(2)) .GT. TOL*ANORM(SM(1),SM(2))) GO TO 21
C
      IF (SN .EQ. 1.D0) GO TO 30
         W(1) = 1.D0 + SM(1)
         W(2) = SM(2)
         RETURN
   30 SM(1) = 0.5D0 + (0.5D0 - SM(1))
      SM(2) = -SM(2)
      IF (MO .EQ. 0) GO TO 140
C
      E = DEXP(SZ(1))
      QF(1) = E*DCOS(SZ(2))
      QF(2) = E*DSIN(SZ(2))
      W(1) = QF(1)*SM(1) - QF(2)*SM(2)
      W(2) = QF(1)*SM(2) + QF(2)*SM(1)
      RETURN
C
C                  TAYLOR SERIES AROUND Z0 = 2
C
   40 IF (SN .LT. 0.D0) GO TO 41
         CALL ERFCM2 (MO, Z, W)
         RETURN
   41 TM(1) = X
      TM(2) = Y
      CALL ERFCM2 (0, TM, W)
      W(1) = 2.D0 - W(1)
      W(2) = - W(2)
      RETURN
C
C            PADE APPROXIMATION FOR THE TAYLOR SERIES
C                    FOR  (EXP(Z*Z)/Z)*ERF(Z)
C
   50 D = 4.D0
      IF (R .GT. 16.D0) D = 16.D0
      IF (R .GT. 64.D0) D = 64.D0
      D2 = D*D
      CALL DCREC (SZ(1), SZ(2), W(1), W(2))
      A0(1) = 1.D0
      A0(2) = 0.D0
      AN(1) = (W(1) + 4.D0/15.D0)*D
      AN(2) = W(2)*D
      B0(1) = 1.D0
      B0(2) = 0.D0
      BN(1) = (W(1) - 0.4D0)*D
      BN(2) = W(2)*D
      CALL CDIVID (AN(1), AN(2), BN(1), BN(2), WN(1), WN(2))
      TOL = 10.D0*EPS
      N4 = 0.D0
C
   60    N4 = N4 + 4.D0
         E = (N4 + 1.D0)*(N4 + 5.D0)
         TM(1) = D*(W(1) - 2.D0/E)
         TM(2) = D*W(2)
         E = D2*(N4*(N4 + 2.0))/((N4 - 1.0)*(N4 + 3.0)*(N4 + 1.0)**2)
C
         QF(1) = (TM(1)*AN(1) - TM(2)*AN(2)) + E*A0(1)
         QF(2) = (TM(1)*AN(2) + TM(2)*AN(1)) + E*A0(2)
         A0(1) = AN(1)
         A0(2) = AN(2)
         AN(1) = QF(1)
         AN(2) = QF(2)
         QF(1) = (TM(1)*BN(1) - TM(2)*BN(2)) + E*B0(1)
         QF(2) = (TM(1)*BN(2) + TM(2)*BN(1)) + E*B0(2)
         B0(1) = BN(1)
         B0(2) = BN(2)
         BN(1) = QF(1)
         BN(2) = QF(2)
C
         W0(1) = WN(1)
         W0(2) = WN(2)
         CALL CDIVID (AN(1), AN(2), BN(1), BN(2), WN(1), WN(2))
         IF (ANORM(WN(1) - W0(1), WN(2) - W0(2)) .GT.
     *          TOL*ANORM(WN(1), WN(2))) GO TO 60
C
      C2 = C + C
      SM(1) = C2*(X*WN(1) - Y*WN(2))
      SM(2) = C2*(X*WN(2) + Y*WN(1))
C
      IF (MO .EQ. 0 .OR. SN .NE. 1.D0) GO TO 70
         E = DEXP(SZ(1))
         W(1) = E*DCOS(SZ(2)) - SM(1)
         W(2) = E*DSIN(SZ(2)) - SM(2)
         RETURN
   70 E = DEXP(-SZ(1))
      QF(1) = E*DCOS(-SZ(2))
      QF(2) = E*DSIN(-SZ(2))
      TM(1) = QF(1)*SM(1) - QF(2)*SM(2)
      TM(2) = QF(1)*SM(2) + QF(2)*SM(1)
      W(1) = 1.D0 - SN*TM(1)
      W(2) = - SN*TM(2)
      RETURN
C
C         PADE APPROXIMATION FOR THE ASYMPTOTIC EXPANSION
C                    FOR  Z*EXP(Z*Z)*ERFC(Z)
C
   80 D = 4.D0*R
      IF (R .LT. 16.D0) D = 16.D0*R
      D2 = D*D
      TM(1) = SZ(1) + SZ(1)
      TM(2) = SZ(2) + SZ(2)
      G0(1) = 1.D0
      G0(2) = 0.D0
      GN(1) = (2.D0 + TM(1))/D
      GN(2) = TM(2)/D
      H0(1) = 1.D0
      H0(2) = 0.D0
      TM(1) = 3.D0 + TM(1)
      HN(1) = TM(1)/D
      HN(2) = TM(2)/D
      CALL CDIVID (GN(1), GN(2), HN(1), HN(2), WN(1), WN(2))
      NP1 = 1.D0
      TOL = 10.D0*EPS
C
   90    N = NP1
         NP1 = N + 1.D0
         N2 = N + N
         E = (N2*(N2 + 1.D0))/D2
         TM(1) = TM(1) + 4.D0
         QF(1) = (TM(1)*GN(1) - TM(2)*GN(2))/D - E*G0(1)
         QF(2) = (TM(1)*GN(2) + TM(2)*GN(1))/D - E*G0(2)
         G0(1) = GN(1)
         G0(2) = GN(2)
         GN(1) = QF(1)
         GN(2) = QF(2)
         QF(1) = (TM(1)*HN(1) - TM(2)*HN(2))/D - E*H0(1)
         QF(2) = (TM(1)*HN(2) + TM(2)*HN(1))/D - E*H0(2)
         H0(1) = HN(1)
         H0(2) = HN(2)
         HN(1) = QF(1)
         HN(2) = QF(2)
C
         W0(1) = WN(1)
         W0(2) = WN(2)
         CALL CDIVID (GN(1), GN(2), HN(1), HN(2), WN(1), WN(2))
         IF (ANORM(WN(1) - W0(1), WN(2) - W0(2)) .GT.
     *          TOL*ANORM(WN(1), WN(2))) GO TO 90
C
      TM(1) = X*HN(1) - Y*HN(2)
      TM(2) = X*HN(2) + Y*HN(1)
      CALL CDIVID (C*GN(1), C*GN(2), TM(1), TM(2), SM(1), SM(2))
      GO TO 130
C
C                      ASYMPTOTIC EXPANSION
C
  100 CALL DCREC (X, Y, TM(1), TM(2))
      SM(1) = TM(1)
      SM(2) = TM(2)
      QF(1) = TM(1)*TM(1) - TM(2)*TM(2)
      QF(2) = 2.D0*TM(1)*TM(2)
      TOL = 2.D0*EPS
      D = -0.5D0
  110    D = D + 1.D0
         TS(1) = TM(1)*QF(1) - TM(2)*QF(2)
         TS(2) = TM(1)*QF(2) + TM(2)*QF(1)
         TM(1) = -D*TS(1)
         TM(2) = -D*TS(2)
         SM(1) = SM(1) + TM(1)
         SM(2) = SM(2) + TM(2)
         IF (ANORM(TM(1),TM(2)) .GT. TOL*ANORM(SM(1),SM(2))) GO TO 110
      SM(1) = C*SM(1)
      SM(2) = C*SM(2)
      IF (X .LT. 1.D-2) GO TO 200
C
C                       TERMINATION
C
  130 IF (MO .NE. 0 .AND. SN .EQ. 1.D0) GO TO 140
      E = DEXP(-SZ(1))
      QF(1) = E*DCOS(-SZ(2))
      QF(2) = E*DSIN(-SZ(2))
      TS(1) = QF(1)*SM(1) - QF(2)*SM(2)
      TS(2) = QF(1)*SM(2) + QF(2)*SM(1)
      SM(1) = TS(1)
      SM(2) = TS(2)
C
      IF (SN .EQ. 1.D0) GO TO 140
         W(1) = 2.D0 - SM(1)
         W(2) = -SM(2)
         RETURN
  140 W(1) = SM(1)
      W(2) = SM(2)
      RETURN
C
C               MODIFIED ASYMPTOTIC EXPANSION
C
  200 IF (MO .NE. 0 .AND. SN .EQ. 1.D0) GO TO 210
      E = DEXP(-SZ(1))
      QF(1) = E*DCOS(-SZ(2))
      QF(2) = E*DSIN(-SZ(2))
      TS(1) = QF(1)*SM(1) - QF(2)*SM(2)
      TS(2) = QF(1)*SM(2) + QF(2)*SM(1)
      W(1) = 1.D0 + SN*TS(1)
      W(2) = SN*TS(2)
      RETURN
C
  210 IF (DABS(Y) .GE. 100.D0) GO TO 140
      IF (SZ(1) .LE. DXPARG) GO TO 140
      E = DEXP(SZ(1))
      W(1) = E*DCOS(SZ(2)) + SM(1)
      W(2) = E*DSIN(SZ(2)) + SM(2)
      RETURN
      END
      SUBROUTINE ERFCM2 (MO, Z, W)
C-----------------------------------------------------------------------
C           CALCULATION OF ERFC(Z) USING THE TAYLOR SERIES
C                          AROUND Z0 = 2
C-----------------------------------------------------------------------
      DOUBLE PRECISION Z(2), W(2)
      DOUBLE PRECISION A(63), C, E, EPS, H(2), T(2), TOL, X, Y
      DOUBLE PRECISION ANORM
C------------------------------
      ANORM(X,Y) = DMAX1(DABS(X),DABS(Y))
C------------------------------
C     C = (2/SQRT(PI))*EXP(-4)
C     E = ERFC(2)
C------------------------------
      DATA C /.20666985354092053857068941306585476D-01/
      DATA E /.46777349810472658379307436327470714D-02/
C------------------------------
      DATA A(1)  / .20000000000000000000000000000000000D+01/,
     *     A(2)  / .23333333333333333333333333333333333D+01/,
     *     A(3)  / .16666666666666666666666666666666667D+01/,
     *     A(4)  / .63333333333333333333333333333333333D+00/,
     *     A(5)  /-.22222222222222222222222222222222222D-01/,
     *     A(6)  /-.16349206349206349206349206349206349D+00/,
     *     A(7)  /-.76984126984126984126984126984126984D-01/,
     *     A(8)  /-.24250440917107583774250440917107584D-02/,
     *     A(9)  / .12716049382716049382716049382716049D-01/,
     *     A(10) / .50208433541766875100208433541766875D-02/
      DATA A(11) /-.25305969750414194858639303083747528D-03/,
     *     A(12) /-.78593217482106370995259884148773038D-03/,
     *     A(13) /-.19118154038788959423880058800693721D-03/,
     *     A(14) / .46324144207742091339974937858535742D-04/,
     *     A(15) / .33885549097189308829520469732109944D-04/,
     *     A(16) / .28637897646612243562134629672756034D-05/,
     *     A(17) /-.29071891082127275370004560446169188D-05/,
     *     A(18) /-.89674405786490646425523560263096103D-06/,
     *     A(19) / .96069103941908684338469767911200105D-07/,
     *     A(20) / .99432863129093191401848891268744113D-07/
      DATA A(21) / .97610310501460621303387795457283579D-08/,
     *     A(22) /-.65557500375673133822289344530892436D-08/,
     *     A(23) /-.18706782059105426900361744016236561D-08/,
     *     A(24) / .20329898993447386223176373714372370D-09/,
     *     A(25) / .16941915827254374668448114614201210D-09/,
     *     A(26) / .10619149520827430973786114446699534D-10/,
     *     A(27) /-.10136148256511788733365237088810952D-10/,
     *     A(28) /-.21042890133669970575386166675721692D-11/,
     *     A(29) / .37186985840699828780916522245407087D-12/,
     *     A(30) / .17921843632701679986488128324051002D-12/
      DATA A(31) /-.89823991804248069863542565948598397D-16/,
     *     A(32) /-.10533182313660970970232171410372199D-13/,
     *     A(33) /-.12340742690978398320850088252659714D-14/,
     *     A(34) / .44315624546581333350568244777175883D-15/,
     *     A(35) / .11584041639989442481950487524296214D-15/,
     *     A(36) /-.10765703619385988116658460442219647D-16/,
     *     A(37) /-.70653158723054941879586082239984222D-17/,
     *     A(38) /-.18708903154917138727191793341667090D-18/,
     *     A(39) / .32549879318817103966053527398133297D-18/,
     *     A(40) / .40654116689599228385911733319215613D-19/
      DATA A(41) /-.11250074516817311101947327325293424D-19/,
     *     A(42) /-.28923865378584966737386008432031980D-20/,
     *     A(43) / .23653053641701517160704870522922706D-21/,
     *     A(44) / .14665384680061888088099002254334292D-21/,
     *     A(45) / .26971039707314316218154193225264469D-23/,
     *     A(46) /-.58753834789274356433279671015522650D-23/,
     *     A(47) /-.59960357240498652932299485494869633D-24/,
     *     A(48) / .18586826578121663981412155416486531D-24/,
     *     A(49) / .38364131854692721721867481914852428D-25/,
     *     A(50) /-.41342210492630142578080062451711039D-26/
      DATA A(51) /-.17646283105274988992381528904600860D-26/,
     *     A(52) / .19828685934364181151988692232131608D-28/,
     *     A(53) / .65592252170840353572672782446212733D-28/,
     *     A(54) / .40626551379996340638338449938639730D-29/,
     *     A(55) /-.20097984104191034713653294173834095D-29/,
     *     A(56) /-.28104226475997460044096389060743131D-30/,
     *     A(57) / .48705319298749358709127987806547949D-31/,
     *     A(58) / .12664655832830787747161769929972617D-31/,
     *     A(59) /-.75168312488894341862391776330113688D-33/,
     *     A(60) /-.45760473722605993842481669806804415D-33/
      DATA A(61) /-.56725491529575395930156379514718000D-35/,
     *     A(62) / .13932664042920082608489441616061541D-34/,
     *     A(63) / .10452448992516358449586503951463322D-35/
C------------------------------
C
C     ****** EPS IS A MACHINE DEPENDENT CONSTANT. EPS IS THE
C            SMALLEST NUMBER SUCH THAT 1.D0 + EPS .GT. 1.D0
C
      DATA EPS /2.220446049250313D-016/
C------------------------------
      TOL = EPS*1.D+12
      H(1) = 1.D0 + (1.D0 - Z(1))
      H(2) = - Z(2)
C
      X = 1.D0
      Y = 0.D0
      W(1) = A(30)
      W(2) = 0.D0
      DO 10 N = 31,63
         T(1) = X*H(1) - Y*H(2)
         T(2) = X*H(2) + Y*H(1)
         X = T(1)
         Y = T(2)
         T(1) = A(N)*X
         T(2) = A(N)*Y
         W(1) = W(1) + T(1)
         W(2) = W(2) + T(2)
         IF (ANORM(T(1),T(2)) .LE. TOL*ANORM(W(1),W(2))) GO TO 20
   10 CONTINUE
C
   20 DO 21 J = 1,29
         N = 30 - J
         X = H(1)*W(1) - H(2)*W(2)
         W(2) = H(1)*W(2) + H(2)*W(1)
         W(1) = A(N) + X
   21 CONTINUE
      X = H(1)*W(1) - H(2)*W(2)
      W(2) = H(1)*W(2) + H(2)*W(1)
      W(1) = 1.D0 + X
C
      X = C*(H(1)*W(1) - H(2)*W(2))
      W(2) = C*(H(1)*W(2) + H(2)*W(1))
      W(1) = E + X
      IF (MO .EQ. 0) RETURN
C
C                     COMPUTE EXP(Z*Z)*ERFC(Z)
C
      X = Z(1)*Z(1) - Z(2)*Z(2)
      Y = 2.D0*Z(1)*Z(2)
      X = DEXP(X)
      T(1) = X*DCOS(Y)
      T(2) = X*DSIN(Y)
      X = T(1)*W(1) - T(2)*W(2)
      Y = T(1)*W(2) + T(2)*W(1)
      W(1) = X
      W(2) = Y
      RETURN
      END
